import "dart:convert";
import "dart:io";
import "package:build/build.dart";

// ignore: implementation_imports
import "package:intl_translation/src/intl_message.dart";
import "package:intl_translation/extract_messages.dart";
import "package:intl_translation/generate_localized.dart";
import "package:path/path.dart" as path;
import "package:l10n/extractor.dart";
import "package:l10n/codegen.dart";
import "package:l10n/arb.dart" as arb;

class ExtractLocaleBuilder implements Builder {
  @override
  Future build(BuildStep buildStep) async {
    final intlMessageExtractor = MessageExtraction();
    final l10nMessageExtractor = L10NMessageExtraction();

    final allMessages = <String, MainMessage>{};
    final intlMessages = intlMessageExtractor.parseFile(File(buildStep.inputId.path));
    allMessages.addAll(intlMessages);

    final l10nMessages = l10nMessageExtractor.parseFile(File(buildStep.inputId.path));
    allMessages.addAll(l10nMessages);

    final localePart = path.basenameWithoutExtension(buildStep.inputId.path);

    final outputPath = path.join(path.dirname(buildStep.inputId.path), "l10n", localePart);
    final mainArbName = "${localePart}.arb";

    arb.writeMessagesToOutputFile(Directory(outputPath), File(mainArbName), allMessages);

    const encoder = JsonEncoder.withIndent("   ");

    for (final locale in ["en", "ru"]) {
      final localeArbName = "${localePart}_${locale}.arb";
      final localeArbPath = path.join(outputPath, localeArbName);
      Map<String, dynamic> prevArbLocale;
      if (FileSystemEntity.isFileSync(localeArbPath)) {
        prevArbLocale = jsonDecoder.decode(File(localeArbPath).readAsStringSync()) as Map<String, dynamic>;
      }
      arb.generateTranslationFile(Directory(outputPath), File(localeArbName), locale, allMessages, true, false);
      if (prevArbLocale != null) {
        final currentArbLocale = jsonDecoder.decode(File(localeArbPath).readAsStringSync()) as Map<String, dynamic>;
        for (final key in currentArbLocale.keys) {
          if (key.startsWith("@")) {
            continue;
          }
          if (prevArbLocale[key] != null) {
            currentArbLocale[key] = prevArbLocale[key];
          }
        }
        File(localeArbPath).writeAsStringSync(encoder.convert(currentArbLocale));
      }
    }

    generateDartCode(
        () => MessageGeneration()
          ..useDeferredLoading = false
          ..codegenMode = "release",
        outputPath,
        outputPath,
        allMessages,
        ["${path.basenameWithoutExtension(buildStep.inputId.path)}.arb"]);

    //await buildStep.writeAsString(outputAsset, jsonEncoder.convert(allMessages));
  }

  @override
  final Map<String, List<String>> buildExtensions = const {
    ".dart": [".arb"]
  };
}

Builder extractLocaleBuilder(BuilderOptions options) => ExtractLocaleBuilder();
